# Copyright (c) HashiCorp, Inc.
# SPDX-License-Identifier: MPL-2.0

variable "client_id" {
  type    = string
  default = "${env("ARM_CLIENT_ID")}"
}

variable "client_secret" {
  type    = string
  default = "${env("ARM_CLIENT_SECRET")}"
}

variable "resource_group_name" {
  type    = string
  default = "${env("ARM_RESOURCE_GROUP_NAME")}"
}

variable "subscription_id" {
  type    = string
  default = "${env("ARM_SUBSCRIPTION_ID")}"
}

variable "resource_prefix" {
  type    = string
  default = "${env("ARM_RESOURCE_PREFIX")}"
}


locals { timestamp = regex_replace(timestamp(), "[- TZ:]", "") }

source "azure-dtl" "autogenerated_1" {
  client_id     = "${var.client_id}"
  client_secret = "${var.client_secret}"
  dtl_artifacts {
    artifact_name = "linux-apt-package"
    parameters {
      name  = "packages"
      value = "vim"
    }
    parameters {
      name  = "update"
      value = "true"
    }
    parameters {
      name  = "options"
      value = "--fix-broken"
    }
  }
  image_offer                       = "UbuntuServer"
  image_publisher                   = "Canonical"
  image_sku                         = "16.04-LTS"
  lab_name                          = "${var.resource_prefix}-packer-acceptance-test"
  lab_resource_group_name           = "${var.resource_group_name}"
  lab_virtual_network_name          = "dtlpacker-acceptance-test"
  location                          = "South Central US"
  managed_image_name                = "testBuilderAccManagedDiskLinux-${local.timestamp}"
  managed_image_resource_group_name = "${var.resource_group_name}"
  os_type                           = "Linux"
  polling_duration_timeout          = "25m"
  subscription_id                   = "${var.subscription_id}"
  vm_size                           = "Standard_DS2_v2"
}

build {
  sources = ["source.azure-dtl.autogenerated_1"]

}
